/////////////////////////////////////////////////////////////////////////////
// GliD3D_Render.cpp : No comment
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
// Directive

#include "GliD3D.h"
#include "gl2ideal.h"

/////////////////////////////////////////////////////////////////////////////

inline void FlushAlphaBlendFunction(void)
{
	if( f_alpha_blend_function_rgb_sf == g_gr_state.alpha_blend_function_rgb_sf &&
		f_alpha_blend_function_rgb_df == g_gr_state.alpha_blend_function_rgb_df)
		return;

	glBlendFunc (src_blendmap[g_gr_state.alpha_blend_function_rgb_sf], 
				dst_blendmap[g_gr_state.alpha_blend_function_rgb_df]);
	f_alpha_blend_function_rgb_sf = g_gr_state.alpha_blend_function_rgb_sf;
	f_alpha_blend_function_rgb_df = g_gr_state.alpha_blend_function_rgb_df;
}

inline void FlushAlphaTestFunction(void)
{
	if( f_alpha_test_function == g_gr_state.alpha_test_function &&
		f_alpha_test_reference_value == g_gr_state.alpha_test_reference_value)
		return;

	glAlphaFunc (GL_NEVER + g_gr_state.alpha_test_function, DIV255(g_gr_state.alpha_test_reference_value));
	f_alpha_test_function = g_gr_state.alpha_test_function;
	f_alpha_test_reference_value = g_gr_state.alpha_test_reference_value;
}

inline void FlushDepthTestFunction(void)
{
	if(f_depth_buffer_function == g_gr_state.depth_buffer_function)
		return;
	glDepthFunc (GL_NEVER + g_gr_state.depth_buffer_function);
	f_depth_buffer_function = g_gr_state.depth_buffer_function;
}

inline void FlushDepthMask(void)
{
	if(f_depth_mask == g_gr_state.depth_mask)
		return;

	glDepthMask (g_gr_state.depth_mask);
	f_depth_mask = g_gr_state.depth_mask;
}

inline void FlushCullMode(void)
{
	if (f_cull_mode == g_gr_state.cull_mode)
		return;

	switch (g_gr_state.cull_mode) {
	case GR_CULL_DISABLE:
		glDisable (GL_CULL_FACE);
		break;
	case GR_CULL_NEGATIVE:
		glCullFace (GL_BACK);
		glEnable (GL_CULL_FACE);
		break;
	case GR_CULL_POSITIVE:
		glCullFace (GL_FRONT);
		glEnable (GL_CULL_FACE);
		break;
	}
	f_cull_mode = g_gr_state.cull_mode;
}

inline void FlushTexEnvi(void)
{
	if(f_texenv == g_gr_texenv)
		return;

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, g_gr_texenv);
	f_texenv = g_gr_texenv;
}

inline void ColorAlphaCombineSetup ()
{
	if(!noTexColor || !noTexAlpha) {
		if(	combine_color_mode == 1 && combine_alpha_mode == 1)
		{
			if(g_gr_bUseTexEnvCombine){
				g_gr_texenv = GL_COMBINE_EXT;
			}else if(g_gr_bUseTexEnvBlend){
				// Do same-color-blending to emulate this
				// It is not perfect. (when local color is iterated)
				g_gr_texenv = GL_BLEND;		// or Reverse_Blend
			}
		}
		FlushTexEnvi();
		EnableTexture2D();
		ChooseVertexFunc();
	}else{
		DisableTexture2D();
		ChooseVertexFunc();
	}
}

inline void FlushDrawState(void)
{
	if(!flushDrawState)
		return;

	flushDrawState = 0;
	ColorAlphaCombineSetup();

	if ((g_gr_state.alpha_blend_function_rgb_sf == GR_BLEND_ONE &&
		 g_gr_state.alpha_blend_function_rgb_df == GR_BLEND_ZERO)){
		DisableBlend();
	}else{
		FlushAlphaBlendFunction();
		EnableBlend();
	}
	if (g_gr_state.alpha_test_function == GR_CMP_ALWAYS ||
		g_gr_state.alpha_combine_other != GR_COMBINE_OTHER_TEXTURE)
		DisableAlphaTest();
	else{
		FlushAlphaTestFunction();
		EnableAlphaTest();
	}
	FlushDepthMask();
	FlushDepthTestFunction();
	FlushCullMode();
}

void InitDrawState(void)
{
	f_texenv = g_gr_texenv;
	f_alpha_blend_function_rgb_sf = g_gr_state.alpha_blend_function_rgb_sf;
	f_alpha_blend_function_rgb_df = g_gr_state.alpha_blend_function_rgb_df;
	f_alpha_test_function = g_gr_state.alpha_test_function;
	f_alpha_test_reference_value = g_gr_state.alpha_test_reference_value;
	f_depth_mask = g_gr_state.depth_mask;
	f_depth_buffer_function = g_gr_state.depth_buffer_function;
	f_cull_mode = g_gr_state.cull_mode;
	flushDrawState = 0;
}


inline void CheckColorMode(const GrVertex * a, bool colorConv = true)
{
	if(combine_color_mode == 0)
		return;

	if( combine_color_mode == 2) {
		const GL2IDE_COLOR	color = { a->r/255, a->g/255, a->b/255, a->a/255};
		SetTexEnvColor(color);
	}else /*if(combine_color_mode == 1)*/{
//		if(!g_gr_bUseTexEnvCombine && combine_alpha_mode == 1) {
		if(g_gr_texenv == GL_BLEND) {
			if(colorConv)
				ColorConvFromGrVertex(ptlv[0].color, a);
			SetTexEnvColor(ptlv[0].color);
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
// Defines / Variables

#define GLID3D_ANTIALIAS_ENABLE
#define GLID3D_ANTIALIAS_DISABLE

/////////////////////////////////////////////////////////////////////////////
// Prototypes

/////////////////////////////////////////////////////////////////////////////
// Implements

FX_ENTRY void FX_CALL grDrawLine (const GrVertex * a, const GrVertex * b)
{
	TRACEFUNCTION ("grDrawLine");

	FlushTriangleVertex();
//	FlushDrawState();
	FlushDepthMask();
	FlushDepthTestFunction();

	grsubSetVertexFromGrVertex (ptlv[0].cord, ptlv[0].color, ptlv[0].tcord, a);
	grsubSetVertexFromGrVertex (ptlv[1].cord, ptlv[1].color, ptlv[1].tcord, b);
	glBegin (GL_LINES);
	glTexCoord2fv ((const GLfloat *)&ptlv[0].tcord);
	glColor4fv ((const GLfloat *)&ptlv[0].color);
	glVertex3fv ((const GLfloat *)&ptlv[0].cord);
	glTexCoord2fv ((const GLfloat *)&ptlv[1].tcord);
	glColor4fv ((const GLfloat *)&ptlv[1].color);
	glVertex3fv ((const GLfloat *)&ptlv[1].cord);
	glEnd ();

}

FX_ENTRY void FX_CALL grDrawPlanarPolygon (int nverts, const int ilist[], const
	GrVertex vlist[])
{
	TRACEFUNCTION ("grDrawPlannarPolygon");

	FlushTriangleVertex();
	FlushDrawState();
	CheckColorMode(&vlist[ilist[0]]);

	int	i;

#ifdef USEVERTEXARRAY
	for (i = 0; i < nverts; i++) {
		grsubSetVertexFromGrVertex (CoordArray[i], ColorArray[i], TexCoordArray[i], &vlist[ilist[i]]);
	}
	glTexCoordPointer (2, GL_FLOAT, 0, TexCoordArray);
	glColorPointer( 4, GL_FLOAT, 0, ColorArray);
	glVertexPointer( 4, GL_FLOAT, 0, CoordArray);
	glDrawArrays( GL_POLYGON, 0, i);
#else
	glBegin (GL_POLYGON);
	for (i = 0; i < nverts; i++) {
		grsubSetVertexFromGrVertex (ptlv[0].cord, ptlv[0].color, ptlv[0].tcord, &vlist[ilist[i]]);
		glTexCoord2fv ((const GLfloat *)&ptlv[0].tcord);
		glColor4fv ((const GLfloat*)&ptlv[0].color);
		glVertex4fv ((const GLfloat*)&ptlv[0].cord);
	}
	glEnd ();
#endif	//USEVERTEXARRAY
}

FX_ENTRY void FX_CALL grDrawPlanarPolygonVertexList (int nverts, const
	GrVertex vlist[])
{
	TRACEFUNCTION ("grPlanarPolygonVertexList");

	grDrawPolygonVertexList (nverts, vlist);
}

FX_ENTRY void FX_CALL grDrawPoint (const GrVertex * pt)
{
	TRACEFUNCTION ("grDrawPoint");

	FlushTriangleVertex();
//	FlushDrawState();
	FlushDepthMask();
	FlushDepthTestFunction();

	grsubSetVertexFromGrVertex (ptlv[0].cord, ptlv[0].color, ptlv[0].tcord, pt);
	glBegin (GL_POINTS);
	glColor4fv ((const GLfloat *)&ptlv[0].color);
	glVertex3fv ((const GLfloat *)&ptlv[0].cord);
	glEnd ();
}

// Untested
FX_ENTRY void FX_CALL grDrawPolygon (int nverts, const int ilist[], const
	GrVertex vlist[])
{
	TRACEFUNCTION ("grDrawPolygon");

	grDrawPlanarPolygon (nverts, ilist, vlist);
}

// Untested
FX_ENTRY void FX_CALL grDrawPolygonVertexList (int nverts, const GrVertex
	vlist[])
{
	TRACEFUNCTION ("grDrawPolygonVertexList");

	FlushTriangleVertex();
	FlushDrawState();
	CheckColorMode(&vlist[0]);

	int	i;

#ifdef USEVERTEXARRAY
	for (i = 0; i < nverts; i++) {
		grsubSetVertexFromGrVertex (CoordArray[i], ColorArray[i], TexCoordArray[i], &vlist[i]);
	}
	glTexCoordPointer (2, GL_FLOAT, 0, TexCoordArray);
	glColorPointer( 4, GL_FLOAT, 0, ColorArray);
	glVertexPointer( 4, GL_FLOAT, 0, CoordArray);
	glDrawArrays( GL_POLYGON, 0, i);
#else
	glBegin (GL_POLYGON);
	for (i = 0; i < nverts; i++) {
		grsubSetVertexFromGrVertex (ptlv[0].cord, ptlv[0].color, ptlv[0].tcord, &vlist[i]);
		glTexCoord2fv ((const GLfloat *)&ptlv[0].tcord);
		glColor4fv ((const GLfloat*)&ptlv[0].color);
		glVertex4fv ((const GLfloat*)&ptlv[0].cord);
	}
	glEnd ();
#endif	//USEVERTEXARRAY
}

FX_ENTRY void FX_CALL grDrawTriangle (const GrVertex * a, const GrVertex *
	 b, const GrVertex * c)
{
	TRACEFUNCTION ("grDrawTriangle");

	FlushDrawState();

#ifdef USEVERTEXARRAY
	CheckColorMode(a);

	grsubSetVertexFromGrVertex(CoordArray[ArrayCount],
	  ColorArray[ArrayCount], TexCoordArray[ArrayCount], a);
	ArrayCount++;
	grsubSetVertexFromGrVertex(CoordArray[ArrayCount],
	  ColorArray[ArrayCount], TexCoordArray[ArrayCount], b);
	ArrayCount++;
	grsubSetVertexFromGrVertex(CoordArray[ArrayCount],
	  ColorArray[ArrayCount], TexCoordArray[ArrayCount], c);
	ArrayCount++;

	if (ArrayCount==MAXVERTEX)
		FlushTriangleVertex();

#else
	grsubSetVertexFromGrVertex(ptlv[0].cord, ptlv[0].color, ptlv[0].tcord, a);
	grsubSetVertexFromGrVertex(ptlv[1].cord, ptlv[1].color, ptlv[1].tcord, b);
	grsubSetVertexFromGrVertex(ptlv[2].cord, ptlv[2].color, ptlv[2].tcord, c);

	CheckColorMode(a, false);

	if(g_gr_DrawingTriangles){
	}else{
		g_gr_DrawingTriangles = TRUE;
		glBegin(GL_TRIANGLES);
	}
	if(g_gr_Texture2DOn){
		glTexCoord2fv ((const GLfloat *)&ptlv[0].tcord);
		glColor4fv ((const GLfloat*)&ptlv[0].color);
		glVertex4fv ((const GLfloat*)&ptlv[0].cord);
		glTexCoord2fv ((const GLfloat *)&ptlv[1].tcord);
		glColor4fv ((const GLfloat*)&ptlv[1].color);
		glVertex4fv ((const GLfloat*)&ptlv[1].cord);
		glTexCoord2fv ((const GLfloat *)&ptlv[2].tcord);
		glColor4fv ((const GLfloat*)&ptlv[2].color);
		glVertex4fv ((const GLfloat*)&ptlv[2].cord);
	}else{
		glColor4fv ((const GLfloat*)&ptlv[0].color);
		glVertex4fv ((const GLfloat*)&ptlv[0].cord);
		glColor4fv ((const GLfloat*)&ptlv[1].color);
		glVertex4fv ((const GLfloat*)&ptlv[1].cord);
		glColor4fv ((const GLfloat*)&ptlv[2].color);
		glVertex4fv ((const GLfloat*)&ptlv[2].cord);
	}
#endif	//USEVERTEXARRAY
}

FX_ENTRY void FX_CALL guAADrawTriangleWithClip (const GrVertex * a, const GrVertex
	 * b, const GrVertex * c)
{
	 TRACEFUNCTION ("guAADrawTriangleWithClip");

	 GLID3D_ANTIALIAS_ENABLE
	 guDrawTriangleWithClip (a, b, c);
	 GLID3D_ANTIALIAS_DISABLE
}

FX_ENTRY void FX_CALL guDrawTriangleWithClip (const GrVertex * a,
	 const GrVertex * b,
	 const GrVertex * c
)
{
	 TRACEFUNCTION ("guDrawTriangleWithClip");
	 grDrawTriangle (a, b, c);
}

FX_ENTRY void FX_CALL guDrawPolygonVertexListWithClip (int nverts, const
	 GrVertex vlist[])
{
	 TRACEFUNCTION ("guDrawPolygonVertexListWithClip");
	 grDrawPolygonVertexList (nverts, vlist);
}

/////////////////////////////////////////////////////////////////////////////
