

#ifndef GL_EXT_texture_env_combine
#define GL_COMBINE_EXT			0x8570
#define GL_COMBINE_RGB_EXT		0x8571
#define GL_COMBINE_ALPHA_EXT	0x8572
#define GL_SOURCE0_RGB_EXT		0x8580
#define GL_SOURCE0_ALPHA_EXT	0x8588
#define GL_SOURCE1_ALPHA_EXT	0x8589
#endif	//GL_EXT_texture_env_combine

#ifndef GL_PRIMARY_COLOR_EXT
#define GL_PRIMARY_COLOR_EXT	0x8577
#endif

#define TEXID_INVALID	0xffffffff
#define DIV255(x) div255_lookuptable[x]
inline void DIV255F(float &x, const float &f){
	if(f < 256.0f)
		x = f*(1.0f/255.0f);
	else
		x = 0.0f;
}

typedef VOID (*vertexconv_func) (CORD &cord, GL2IDE_COLOR &color,
       GL2IDE_TEXCORD &texcord, const GrVertex * pv);

void InitBlendMap(void);
void InitDiv255LookupTable(void);

HWND
CreateOpenGLWindow (HWND Parent_hWnd, char *title, int x, int y, int width, int height,
    BYTE type, DWORD flags);

void InitDrawState(void);
void InitGLExt(void);

/////////////////////////////////////////////////////////////////////////////

inline void ColorConvFromGrVertex (GL2IDE_COLOR &color, const GrVertex * pv)
{
	if(static_color_mode == GR_COMBINE_LOCAL_ITERATED){
		DIV255F(color.r, pv->r);
		DIV255F(color.g, pv->g);
		DIV255F(color.b, pv->b);
		if(g_gr_state.alpha_combine_local == GR_COMBINE_LOCAL_ITERATED){
			DIV255F(color.a, pv->a);
		}else{
			color.a = static_color.a;
		}
	}else{
		color = static_color;
		if(g_gr_state.alpha_combine_local == GR_COMBINE_LOCAL_ITERATED)
			DIV255F(color.a, pv->a);
	}
}

inline void InitLookupVertexFuncs (void){}
inline void ChooseVertexFunc (void){}
inline void grsubSetVertexFromGrVertex (CORD &cord, GL2IDE_COLOR &color,
       GL2IDE_TEXCORD &tcord, const GrVertex * pv)
{
	if (pv->y < 4096.0f)
		cord.y = pv->y;
	else
		cord.y = pv->y - (FLOAT) (((DWORD) pv->y) & 0xfffff000);

	if (pv->x < 4096.0f)
		cord.x = pv->x;
	else
		cord.x = pv->x - (FLOAT) (((DWORD) pv->x) & 0xfffff000);

	cord.w = 1.0f / pv->oow;
		 // scale texture coordinates according to aspect ratio
	if(g_gr_Texture2DOn){
		tcord.tu = pv->tmuvtx[0].sow * cord.w * current_texture->ah;
		tcord.tv = pv->tmuvtx[0].tow * cord.w * current_texture->aw;
	}

	if (g_gr_bUseZInverse) {
		if (g_gr_bPrecisionFix) {
			if (cord.w < 10)
				cord.w = 10;
		}
		cord.z = 1.0f;
	}else{
		cord.z = -cord.w * cord.w;
	}
	cord.x *= cord.w;
	cord.y *= cord.w;

	ColorConvFromGrVertex(color, pv);
}


inline void EnableBlend(void)
{
	if(g_gr_BlendOn)
		return;
	glEnable(GL_BLEND);
	g_gr_BlendOn = TRUE;
}

inline void DisableBlend(void)
{
	if(!g_gr_BlendOn)
		return;
	glDisable(GL_BLEND);
	g_gr_BlendOn = FALSE;
}

inline void EnableAlphaTest(void)
{
	if(g_gr_AlphaTestOn)
		return;
	glEnable(GL_ALPHA_TEST);
	g_gr_AlphaTestOn = TRUE;
}

inline void DisableAlphaTest(void)
{
	if(!g_gr_AlphaTestOn)
		return;
	glDisable(GL_ALPHA_TEST);
	g_gr_AlphaTestOn = FALSE;
}

inline void EnableTexture2D(void)
{
	if(g_gr_Texture2DOn)
		return;
	glEnable(GL_TEXTURE_2D);
	g_gr_Texture2DOn = TRUE;
}

inline void DisableTexture2D(void)
{
	if(!g_gr_Texture2DOn)
		return;
	glDisable(GL_TEXTURE_2D);
	g_gr_Texture2DOn = FALSE;
}

inline void SetTexEnvColor(const GL2IDE_COLOR &col)
{
	if (texEnvColor.r == col.r && texEnvColor.g == col.g && 
		texEnvColor.b == col.b && texEnvColor.a == col.a)
		return;

	texEnvColor = col;
	FlushTriangleVertex();
	glTexEnvfv(GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, &col.r);

	return;
}


/////////////////////////////////////////////////////////////////////////////

inline void InitCombineSetup ()
{
	static_color_mode = g_gr_state.color_combine_local;
	static_color = g_gr_state.constant_color;
	noTexColor = 1;
	noTexAlpha = 1;
	combine_color_mode = 0;
	combine_alpha_mode = 0;

	const GL2IDE_COLOR	white_col = {1,1,1,1};
	texEnvColor.r = 0;
	SetTexEnvColor(white_col);	// set default color
}

inline void i_grAlphaCombine (GrCombineFunction_t function,
    GrCombineFactor_t factor, GrCombineLocal_t local, GrCombineOther_t other,
    FxBool invert)
{
    g_gr_state.alpha_combine_function = function;
    g_gr_state.alpha_combine_factor = factor;
    g_gr_state.alpha_combine_local = local;
    g_gr_state.alpha_combine_other = other;
//    g_gr_state.alpha_combine_invert = invert;

	noTexAlpha = (function == GR_COMBINE_FUNCTION_ZERO ||
		factor == GR_COMBINE_FACTOR_NONE ||
		(factor != GR_COMBINE_FACTOR_TEXTURE_RGB && other != GR_COMBINE_OTHER_TEXTURE));

	if (function == GR_COMBINE_FUNCTION_SCALE_OTHER &&
		other == GR_COMBINE_OTHER_TEXTURE)
	{
		if (factor == GR_COMBINE_FACTOR_ONE)
			static_color.a = 1;
		else if (factor == GR_COMBINE_FACTOR_ZERO)
			static_color.a = 0;
		else
			static_color.a = g_gr_state.constant_color.a;

		combine_alpha_mode = 1;
	}else{
		static_color.a = g_gr_state.constant_color.a;
		combine_alpha_mode = 0;
	}
}

inline void i_grColorCombine (GrCombineFunction_t function,
    GrCombineFactor_t factor, GrCombineLocal_t local, GrCombineOther_t other,
    FxBool invert)
{
    g_gr_state.color_combine_function = function;
    g_gr_state.color_combine_factor = factor;
    g_gr_state.color_combine_local = local;
    g_gr_state.color_combine_other = other;
//    g_gr_state.color_combine_invert = invert;

	g_gr_texenv = GL_MODULATE;
	combine_color_mode = 0;
	static_color_mode = local;

	static_color.r = g_gr_state.constant_color.r;
	static_color.g = g_gr_state.constant_color.g;
	static_color.b = g_gr_state.constant_color.b;
	
	noTexColor = (function == GR_COMBINE_FUNCTION_ZERO ||
		factor == GR_COMBINE_FACTOR_NONE ||
		(factor != GR_COMBINE_FACTOR_TEXTURE_RGB &&
		other != GR_COMBINE_OTHER_TEXTURE));

	if(factor == GR_COMBINE_FACTOR_TEXTURE_RGB){
		if(g_gr_bUseTexEnvBlend){
			g_gr_texenv = GL_BLEND;
			if(	local == GR_COMBINE_LOCAL_CONSTANT &&
				other == GR_COMBINE_OTHER_ITERATED)
			{
				combine_color_mode = 2;
			}
		}else{
			if(other == GR_COMBINE_OTHER_ITERATED)
				static_color_mode = GR_COMBINE_LOCAL_ITERATED;
		}
	}else
	if (function == GR_COMBINE_FUNCTION_SCALE_OTHER &&
		other == GR_COMBINE_OTHER_TEXTURE){
		if(factor == GR_COMBINE_FACTOR_ONE){
			static_color.r = static_color.g = static_color.b = 1;
		}else if(factor == GR_COMBINE_FACTOR_ZERO){
			static_color.r = static_color.g = static_color.b = 0;
		}
	}else
	if(	function == GR_COMBINE_FUNCTION_LOCAL &&
		factor == GR_COMBINE_FACTOR_ONE &&
		local == GR_COMBINE_LOCAL_ITERATED &&
		other == GR_COMBINE_OTHER_CONSTANT){
		combine_color_mode = 1;
	}else
	if (function == GR_COMBINE_FUNCTION_SCALE_OTHER_ADD_LOCAL &&
		local == GR_COMBINE_LOCAL_ITERATED &&
		factor == GR_COMBINE_FACTOR_ONE)
	{
		if(g_gr_bUseTexEnvAdd){
			g_gr_texenv = GL_ADD;
		}else{
			static_color.r = static_color.g = static_color.b = 1;
			static_color_mode = GR_COMBINE_LOCAL_CONSTANT;
		}
	}else
	if (function == GR_COMBINE_FUNCTION_SCALE_OTHER_MINUS_LOCAL_ADD_LOCAL &&
		factor == GR_COMBINE_FACTOR_TEXTURE_ALPHA &&
		other == GR_COMBINE_OTHER_TEXTURE &&
		local == GR_COMBINE_LOCAL_ITERATED){
		g_gr_texenv = GL_DECAL;
	}
}


/////////////////////////////////////////////////////////////////////////////

